//***********************************************************
//  AVR-Synthesizer Version 1.00
// ------------------------------
// 2x DCO with Sinus/ Saw / FM 
// 1x DCA / 3x LFO / 2 ADSR-Generator
// 1x extern VCF with Remote Frequenz and Resonanz
//
//	ATxmega128A1 with 32MHz
//
//  Created: 03.08.2012
//  Author: Rolf Degen
//
//***********************************************************

#define  F_CPU 32000000UL	// Prozessor-Clock 32MHz
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

extern void sound_out(void);   // Interrupt Soundroutine in Assembler 


// LED Port
#define LED_OUT PORTE.OUT	// namen for output PORTE
#define LED_DDR PORTE.DIR	// name for port-direction
#define LED_0 PIN0			// namen for Port-Bit0
#define LED_1 PIN1
#define LED_2 PIN2
#define LED_3 PIN3
#define LED_4 PIN4
#define LED_5 PIN5


// Variable for Sound
uint8_t sound_flag1 = 0;
uint8_t sound_flag2 = 0;
uint8_t tasten_flag1 = 0;
uint16_t frequenz;
uint16_t korrekturwert;		// Korrekturwert fr genau Frequenz der Midi-Note
volatile __uint24 phaccu1 = 0;
volatile __uint24 phaccu2 = 0;
volatile __uint24 schrittweite1 = 0;
volatile __uint24 schrittweite2 = 0;
volatile uint16_t sample1;
volatile uint16_t sample2;
volatile __uint24 sample_x;
volatile uint8_t DcoWaveForm = 0x00;
uint8_t frq_tune = 0;
volatile uint8_t LevelPWM1 = 255;
volatile __uint24 noise = 1907;  // random value for starting noise

__uint24 result;

uint32_t sample_y;
volatile uint16_t phaccu_x;


volatile uint16_t adsr_1 = 0;
uint16_t adsr_1_temp;
uint16_t adsr_2 = 0;

// Variable for ADSR-DCA
uint8_t adsr_phase1 = 0; // ADSR-Phase (1=Attack 2=decay 3=subtain 4=release)
uint16_t attack_level = 0;
uint16_t attack_time = 0;
uint16_t sustain_level = 0;
uint16_t decay_time = 0;
uint16_t release_time = 0;
uint16_t adsr_out = 0;

// Variable for ADSR-VCF
uint8_t adsr_phase1_f = 0; // ADSR-Phase (1=Attack 2=decay 3=subtain 4=release)
uint16_t attack_level_f = 0;
uint16_t attack_time_f = 0;
uint16_t sustain_level_f = 0;
uint16_t decay_time_f = 0;
uint16_t release_time_f = 0;
uint16_t adsr_out_f = 0;
uint8_t resonace_f = 0;

// Variable for LFO_1 DCO
uint16_t lfo1_phaccu = 0;
uint16_t lfo1_sample = 0;
uint16_t lfo1_sample_old = 0;
uint16_t lfo1_time = 0;
uint16_t lfo1_level = 0;
uint8_t  lfo1_phase = 0;
uint32_t lfo1_out = 0;
uint8_t  lfo1_flag = 0;

// Variable for LFO_2 VCF
uint16_t lfo2_phaccu = 0;
uint16_t lfo2_time = 0;
uint8_t  lfo2_level = 0;
uint8_t  lfo2_phase = 0;
uint8_t  lfo2_out = 0;
uint8_t  lfo2_out_tmp = 0;
uint8_t  lfo2_out_old = 0;
uint8_t  lfo2_flag = 0;

// Variable for LFO_3 DCA
uint16_t lfo3_phaccu = 0;
uint16_t lfo3_time = 0;
uint8_t  lfo3_level = 0;
uint8_t  lfo3_phase = 0;
uint16_t  lfo3_out = 0;
uint8_t  lfo3_out_tmp = 0;
uint8_t  lfo3_out_old = 0;
uint8_t  lfo3_flag = 0;

// Variable for Midi 
volatile uint8_t midibuf[256];			// Receive Buffer for Midi-Data
volatile uint8_t midibuf_wr_pointer=0;
volatile uint8_t midibuf_rd_pointer=0;
uint8_t midi_avail = 0; // 0= no mididata in buffer / 1= mididata in buffer
uint8_t midi_data = 0;
uint8_t noten_nr = 0;
uint8_t velocity1 = 0;
uint8_t velocity2 = 0;
uint8_t led_timer = 0;
uint8_t ctrl_data0 = 0;
uint8_t ctrl_data1 = 1;
uint8_t prgchange_nr = 0;
uint8_t arpeggio_flag = 0;
uint16_t arpeggio_timer0 = 250;
uint16_t arpeggio_timer1 = 50;
uint8_t arpeggio_step = 0;


// 8Bit Sinus-Table
const uint8_t sinus[] PROGMEM = {
	127, 130, 134, 137, 140, 143, 146, 149,
	152, 155, 158, 162, 165, 167, 170, 173,
	176, 179, 182, 185, 188, 190, 193, 196,
	198, 201, 203, 206, 208, 211, 213, 215,
	218, 220, 222, 224, 226, 228, 230, 232,
	234, 235, 237, 238, 240, 241, 243, 244,
	245, 246, 248, 249, 250, 250, 251, 252,
	253, 253, 254, 254, 254, 255, 255, 255,
	255, 255, 255, 255, 254, 254, 254, 253,
	253, 252, 251, 250, 250, 249, 248, 246,
	245, 244, 243, 241, 240, 238, 237, 235,
	234, 232, 230, 228, 226, 224, 222, 220,
	218, 215, 213, 211, 208, 206, 203, 201,
	198, 196, 193, 190, 188, 185, 182, 179,
	176, 173, 170, 167, 165, 162, 158, 155,
	152, 149, 146, 143, 140, 137, 134, 131,
	128, 124, 121, 118, 115, 112, 109, 106,
	103, 100, 97, 93, 90, 88, 85, 82,
	79, 76, 73, 70, 67, 65, 62, 59,
	57, 54, 52, 49, 47, 44, 42, 40,
	37, 35, 33, 31, 29, 27, 25, 23,
	21, 20, 18, 17, 15, 14, 12, 11,
	10, 9, 7, 6, 5, 5, 4, 3,
	2, 2, 1, 1, 1, 0, 0, 0,
	0, 0, 0, 0, 1, 1, 1, 2,
	2, 3, 4, 5, 5, 6, 7, 9,
	10, 11, 12, 14, 15, 17, 18, 20,
	21, 23, 25, 27, 29, 31, 33, 35,
	37, 40, 42, 44, 47, 49, 52, 54,
	57, 59, 62, 65, 67, 70, 73, 76,
	79, 82, 85, 88, 90, 93, 97, 100,
	103, 106, 109, 112, 115, 118, 121, 124
};
// 8Bit Rectangle-Table
const uint8_t rectangle[] PROGMEM = {
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255
};
// 8Bit Triangle-Table
const uint8_t triangle[] PROGMEM = {
	0, 2, 4, 6, 8, 10, 12, 14,
	16, 18, 20, 22, 24, 26, 28, 30,
	32, 34, 36, 38, 40, 42, 44, 46,
	48, 50, 52, 54, 56, 58, 60, 62,
	64, 66, 68, 70, 72, 74, 76, 78,
	80, 82, 84, 86, 88, 90, 92, 94,
	96, 98, 100, 102, 104, 106, 108, 110,
	112, 114, 116, 118, 120, 122, 124, 126,
	128, 130, 132, 134, 136, 138, 140, 142,
	144, 146, 148, 150, 152, 154, 156, 158,
	160, 162, 164, 166, 168, 170, 172, 174,
	176, 178, 180, 182, 184, 186, 188, 190,
	192, 194, 196, 198, 200, 202, 204, 206,
	208, 210, 212, 214, 216, 218, 220, 222,
	224, 226, 228, 230, 232, 234, 236, 238,
	240, 242, 244, 246, 248, 250, 252, 254,
	255, 253, 251, 249, 247, 245, 243, 241,
	239, 237, 235, 233, 231, 229, 227, 225,
	223, 221, 219, 217, 215, 213, 211, 209,
	207, 205, 203, 201, 199, 197, 195, 193,
	191, 189, 187, 185, 183, 181, 179, 177,
	175, 173, 171, 169, 167, 165, 163, 161,
	159, 157, 155, 153, 151, 149, 147, 145,
	143, 141, 139, 137, 135, 133, 131, 129,
	127, 125, 123, 121, 119, 117, 115, 113,
	111, 109, 107, 105, 103, 101, 99, 97,
	95, 93, 91, 89, 87, 85, 83, 81,
	79, 77, 75, 73, 71, 69, 67, 65,
	63, 61, 59, 57, 55, 53, 51, 49,
	47, 45, 43, 41, 39, 37, 35, 33,
	31, 29, 27, 25, 23, 21, 19, 17,
	15, 13, 11, 9, 7, 5, 3, 1
};
// 8Bit Sawtooth-Table
const uint8_t saw[] PROGMEM = {
	0, 1, 2, 3, 4, 5, 6, 7,
	8, 9, 10, 11, 12, 13, 14, 15,
	16, 17, 18, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39,
	40, 41, 42, 43, 44, 45, 46, 47,
	48, 49, 50, 51, 52, 53, 54, 55,
	56, 57, 58, 59, 60, 61, 62, 63,
	64, 65, 66, 67, 68, 69, 70, 71,
	72, 73, 74, 75, 76, 77, 78, 79,
	80, 81, 82, 83, 84, 85, 86, 87,
	88, 89, 90, 91, 92, 93, 94, 95,
	96, 97, 98, 99, 100, 101, 102, 103,
	104, 105, 106, 107, 108, 109, 110, 111,
	112, 113, 114, 115, 116, 117, 118, 119,
	120, 121, 122, 123, 124, 125, 126, 127,
	128, 129, 130, 131, 132, 133, 134, 135,
	136, 137, 138, 139, 140, 141, 142, 143,
	144, 145, 146, 147, 148, 149, 150, 151,
	152, 153, 154, 155, 156, 157, 158, 159,
	160, 161, 162, 163, 164, 165, 166, 167,
	168, 169, 170, 171, 172, 173, 174, 175,
	176, 177, 178, 179, 180, 181, 182, 183,
	184, 185, 186, 187, 188, 189, 190, 191,
	192, 193, 194, 195, 196, 197, 198, 199,
	200, 201, 202, 203, 204, 205, 206, 207,
	208, 209, 210, 211, 212, 213, 214, 215,
	216, 217, 218, 219, 220, 221, 222, 223,
	224, 225, 226, 227, 228, 229, 230, 231,
	232, 233, 234, 235, 236, 237, 238, 239,
	240, 241, 242, 243, 244, 245, 246, 247,
	248, 249, 250, 251, 252, 253, 254, 255
};


// 12Bit Sinus-Tabelle mit 4096 Werten
const uint16_t sinus_12Bit[] PROGMEM = {     
      2047, 2050, 2053, 2056, 2060, 2063, 2066, 2069, 2072, 2075, 2078, 2082, 2085, 2088, 2091, 2094, 2097, 2100, 2104, 2107,
      2110, 2113, 2116, 2119, 2122, 2126, 2129, 2132, 2135, 2138, 2141, 2144, 2147, 2151, 2154, 2157, 2160, 2163, 2166, 2169,
      2173, 2176, 2179, 2182, 2185, 2188, 2191, 2195, 2198, 2201, 2204, 2207, 2210, 2213, 2216, 2220, 2223, 2226, 2229, 2232,
      2235, 2238, 2241, 2245, 2248, 2251, 2254, 2257, 2260, 2263, 2266, 2270, 2273, 2276, 2279, 2282, 2285, 2288, 2291, 2295,
      2298, 2301, 2304, 2307, 2310, 2313, 2316, 2320, 2323, 2326, 2329, 2332, 2335, 2338, 2341, 2344, 2348, 2351, 2354, 2357,
      2360, 2363, 2366, 2369, 2372, 2375, 2379, 2382, 2385, 2388, 2391, 2394, 2397, 2400, 2403, 2406, 2410, 2413, 2416, 2419,
      2422, 2425, 2428, 2431, 2434, 2437, 2440, 2443, 2447, 2450, 2453, 2456, 2459, 2462, 2465, 2468, 2471, 2474, 2477, 2480,
      2483, 2487, 2490, 2493, 2496, 2499, 2502, 2505, 2508, 2511, 2514, 2517, 2520, 2523, 2526, 2529, 2532, 2535, 2539, 2542,
      2545, 2548, 2551, 2554, 2557, 2560, 2563, 2566, 2569, 2572, 2575, 2578, 2581, 2584, 2587, 2590, 2593, 2596, 2599, 2602,
      2605, 2608, 2611, 2614, 2617, 2620, 2623, 2626, 2629, 2632, 2635, 2638, 2642, 2645, 2648, 2651, 2654, 2657, 2660, 2663,
      2666, 2669, 2671, 2674, 2677, 2680, 2683, 2686, 2689, 2692, 2695, 2698, 2701, 2704, 2707, 2710, 2713, 2716, 2719, 2722,
      2725, 2728, 2731, 2734, 2737, 2740, 2743, 2746, 2749, 2752, 2755, 2758, 2761, 2764, 2766, 2769, 2772, 2775, 2778, 2781,
      2784, 2787, 2790, 2793, 2796, 2799, 2802, 2805, 2807, 2810, 2813, 2816, 2819, 2822, 2825, 2828, 2831, 2834, 2837, 2839,
      2842, 2845, 2848, 2851, 2854, 2857, 2860, 2863, 2865, 2868, 2871, 2874, 2877, 2880, 2883, 2886, 2888, 2891, 2894, 2897,
      2900, 2903, 2906, 2908, 2911, 2914, 2917, 2920, 2923, 2925, 2928, 2931, 2934, 2937, 2940, 2942, 2945, 2948, 2951, 2954,
      2957, 2959, 2962, 2965, 2968, 2971, 2973, 2976, 2979, 2982, 2985, 2987, 2990, 2993, 2996, 2999, 3001, 3004, 3007, 3010,
      3012, 3015, 3018, 3021, 3023, 3026, 3029, 3032, 3035, 3037, 3040, 3043, 3046, 3048, 3051, 3054, 3056, 3059, 3062, 3065,
      3067, 3070, 3073, 3076, 3078, 3081, 3084, 3086, 3089, 3092, 3094, 3097, 3100, 3103, 3105, 3108, 3111, 3113, 3116, 3119,
      3121, 3124, 3127, 3129, 3132, 3135, 3137, 3140, 3143, 3145, 3148, 3151, 3153, 3156, 3159, 3161, 3164, 3166, 3169, 3172,
      3174, 3177, 3180, 3182, 3185, 3187, 3190, 3193, 3195, 3198, 3200, 3203, 3206, 3208, 3211, 3213, 3216, 3219, 3221, 3224,
      3226, 3229, 3231, 3234, 3237, 3239, 3242, 3244, 3247, 3249, 3252, 3254, 3257, 3259, 3262, 3264, 3267, 3270, 3272, 3275,
      3277, 3280, 3282, 3285, 3287, 3290, 3292, 3295, 3297, 3300, 3302, 3305, 3307, 3309, 3312, 3314, 3317, 3319, 3322, 3324,
      3327, 3329, 3332, 3334, 3336, 3339, 3341, 3344, 3346, 3349, 3351, 3354, 3356, 3358, 3361, 3363, 3366, 3368, 3370, 3373,
      3375, 3378, 3380, 3382, 3385, 3387, 3389, 3392, 3394, 3397, 3399, 3401, 3404, 3406, 3408, 3411, 3413, 3415, 3418, 3420,
      3422, 3425, 3427, 3429, 3432, 3434, 3436, 3439, 3441, 3443, 3445, 3448, 3450, 3452, 3455, 3457, 3459, 3461, 3464, 3466,
      3468, 3471, 3473, 3475, 3477, 3480, 3482, 3484, 3486, 3488, 3491, 3493, 3495, 3497, 3500, 3502, 3504, 3506, 3508, 3511,
      3513, 3515, 3517, 3519, 3522, 3524, 3526, 3528, 3530, 3532, 3535, 3537, 3539, 3541, 3543, 3545, 3547, 3550, 3552, 3554,
      3556, 3558, 3560, 3562, 3564, 3567, 3569, 3571, 3573, 3575, 3577, 3579, 3581, 3583, 3585, 3587, 3590, 3592, 3594, 3596,
      3598, 3600, 3602, 3604, 3606, 3608, 3610, 3612, 3614, 3616, 3618, 3620, 3622, 3624, 3626, 3628, 3630, 3632, 3634, 3636,
      3638, 3640, 3642, 3644, 3646, 3648, 3650, 3652, 3654, 3656, 3658, 3660, 3662, 3663, 3665, 3667, 3669, 3671, 3673, 3675,
      3677, 3679, 3681, 3683, 3684, 3686, 3688, 3690, 3692, 3694, 3696, 3698, 3699, 3701, 3703, 3705, 3707, 3709, 3710, 3712,
      3714, 3716, 3718, 3720, 3721, 3723, 3725, 3727, 3729, 3730, 3732, 3734, 3736, 3738, 3739, 3741, 3743, 3745, 3746, 3748,
      3750, 3752, 3753, 3755, 3757, 3759, 3760, 3762, 3764, 3765, 3767, 3769, 3771, 3772, 3774, 3776, 3777, 3779, 3781, 3782,
      3784, 3786, 3787, 3789, 3791, 3792, 3794, 3796, 3797, 3799, 3800, 3802, 3804, 3805, 3807, 3808, 3810, 3812, 3813, 3815,
      3816, 3818, 3820, 3821, 3823, 3824, 3826, 3827, 3829, 3830, 3832, 3834, 3835, 3837, 3838, 3840, 3841, 3843, 3844, 3846,
      3847, 3849, 3850, 3852, 3853, 3855, 3856, 3858, 3859, 3861, 3862, 3863, 3865, 3866, 3868, 3869, 3871, 3872, 3873, 3875,
      3876, 3878, 3879, 3881, 3882, 3883, 3885, 3886, 3887, 3889, 3890, 3892, 3893, 3894, 3896, 3897, 3898, 3900, 3901, 3902,
      3904, 3905, 3906, 3908, 3909, 3910, 3912, 3913, 3914, 3915, 3917, 3918, 3919, 3921, 3922, 3923, 3924, 3926, 3927, 3928,
      3929, 3931, 3932, 3933, 3934, 3935, 3937, 3938, 3939, 3940, 3942, 3943, 3944, 3945, 3946, 3947, 3949, 3950, 3951, 3952,
      3953, 3954, 3955, 3957, 3958, 3959, 3960, 3961, 3962, 3963, 3964, 3966, 3967, 3968, 3969, 3970, 3971, 3972, 3973, 3974,
      3975, 3976, 3977, 3978, 3979, 3981, 3982, 3983, 3984, 3985, 3986, 3987, 3988, 3989, 3990, 3991, 3992, 3993, 3994, 3995,
      3996, 3997, 3997, 3998, 3999, 4000, 4001, 4002, 4003, 4004, 4005, 4006, 4007, 4008, 4009, 4010, 4010, 4011, 4012, 4013,
      4014, 4015, 4016, 4017, 4017, 4018, 4019, 4020, 4021, 4022, 4022, 4023, 4024, 4025, 4026, 4027, 4027, 4028, 4029, 4030,
      4031, 4031, 4032, 4033, 4034, 4034, 4035, 4036, 4037, 4037, 4038, 4039, 4040, 4040, 4041, 4042, 4042, 4043, 4044, 4045,
      4045, 4046, 4047, 4047, 4048, 4049, 4049, 4050, 4051, 4051, 4052, 4053, 4053, 4054, 4054, 4055, 4056, 4056, 4057, 4057,
      4058, 4059, 4059, 4060, 4060, 4061, 4062, 4062, 4063, 4063, 4064, 4064, 4065, 4065, 4066, 4066, 4067, 4067, 4068, 4068,
      4069, 4069, 4070, 4070, 4071, 4071, 4072, 4072, 4073, 4073, 4074, 4074, 4075, 4075, 4076, 4076, 4076, 4077, 4077, 4078,
      4078, 4078, 4079, 4079, 4080, 4080, 4080, 4081, 4081, 4081, 4082, 4082, 4083, 4083, 4083, 4084, 4084, 4084, 4085, 4085,
      4085, 4085, 4086, 4086, 4086, 4087, 4087, 4087, 4087, 4088, 4088, 4088, 4088, 4089, 4089, 4089, 4089, 4090, 4090, 4090,
      4090, 4091, 4091, 4091, 4091, 4091, 4092, 4092, 4092, 4092, 4092, 4092, 4093, 4093, 4093, 4093, 4093, 4093, 4093, 4093,
      4094, 4094, 4094, 4094, 4094, 4094, 4094, 4094, 4094, 4094, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095,
      4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4095, 4094,
      4094, 4094, 4094, 4094, 4094, 4094, 4094, 4094, 4094, 4093, 4093, 4093, 4093, 4093, 4093, 4093, 4093, 4092, 4092, 4092,
      4092, 4092, 4092, 4091, 4091, 4091, 4091, 4091, 4090, 4090, 4090, 4090, 4089, 4089, 4089, 4089, 4088, 4088, 4088, 4088,
      4087, 4087, 4087, 4087, 4086, 4086, 4086, 4085, 4085, 4085, 4085, 4084, 4084, 4084, 4083, 4083, 4083, 4082, 4082, 4081,
      4081, 4081, 4080, 4080, 4080, 4079, 4079, 4078, 4078, 4078, 4077, 4077, 4076, 4076, 4076, 4075, 4075, 4074, 4074, 4073,
      4073, 4072, 4072, 4071, 4071, 4070, 4070, 4069, 4069, 4068, 4068, 4067, 4067, 4066, 4066, 4065, 4065, 4064, 4064, 4063,
      4063, 4062, 4062, 4061, 4060, 4060, 4059, 4059, 4058, 4057, 4057, 4056, 4056, 4055, 4054, 4054, 4053, 4053, 4052, 4051,
      4051, 4050, 4049, 4049, 4048, 4047, 4047, 4046, 4045, 4045, 4044, 4043, 4042, 4042, 4041, 4040, 4040, 4039, 4038, 4037,
      4037, 4036, 4035, 4034, 4034, 4033, 4032, 4031, 4031, 4030, 4029, 4028, 4027, 4027, 4026, 4025, 4024, 4023, 4022, 4022,
      4021, 4020, 4019, 4018, 4017, 4017, 4016, 4015, 4014, 4013, 4012, 4011, 4010, 4010, 4009, 4008, 4007, 4006, 4005, 4004,
      4003, 4002, 4001, 4000, 3999, 3998, 3997, 3997, 3996, 3995, 3994, 3993, 3992, 3991, 3990, 3989, 3988, 3987, 3986, 3985,
      3984, 3983, 3982, 3981, 3979, 3978, 3977, 3976, 3975, 3974, 3973, 3972, 3971, 3970, 3969, 3968, 3967, 3966, 3964, 3963,
      3962, 3961, 3960, 3959, 3958, 3957, 3955, 3954, 3953, 3952, 3951, 3950, 3949, 3947, 3946, 3945, 3944, 3943, 3942, 3940,
      3939, 3938, 3937, 3935, 3934, 3933, 3932, 3931, 3929, 3928, 3927, 3926, 3924, 3923, 3922, 3921, 3919, 3918, 3917, 3915,
      3914, 3913, 3912, 3910, 3909, 3908, 3906, 3905, 3904, 3902, 3901, 3900, 3898, 3897, 3896, 3894, 3893, 3892, 3890, 3889,
      3887, 3886, 3885, 3883, 3882, 3881, 3879, 3878, 3876, 3875, 3873, 3872, 3871, 3869, 3868, 3866, 3865, 3863, 3862, 3861,
      3859, 3858, 3856, 3855, 3853, 3852, 3850, 3849, 3847, 3846, 3844, 3843, 3841, 3840, 3838, 3837, 3835, 3834, 3832, 3830,
      3829, 3827, 3826, 3824, 3823, 3821, 3820, 3818, 3816, 3815, 3813, 3812, 3810, 3808, 3807, 3805, 3804, 3802, 3800, 3799,
      3797, 3796, 3794, 3792, 3791, 3789, 3787, 3786, 3784, 3782, 3781, 3779, 3777, 3776, 3774, 3772, 3771, 3769, 3767, 3765,
      3764, 3762, 3760, 3759, 3757, 3755, 3753, 3752, 3750, 3748, 3746, 3745, 3743, 3741, 3739, 3738, 3736, 3734, 3732, 3730,
      3729, 3727, 3725, 3723, 3721, 3720, 3718, 3716, 3714, 3712, 3710, 3709, 3707, 3705, 3703, 3701, 3699, 3698, 3696, 3694,
      3692, 3690, 3688, 3686, 3684, 3683, 3681, 3679, 3677, 3675, 3673, 3671, 3669, 3667, 3665, 3663, 3662, 3660, 3658, 3656,
      3654, 3652, 3650, 3648, 3646, 3644, 3642, 3640, 3638, 3636, 3634, 3632, 3630, 3628, 3626, 3624, 3622, 3620, 3618, 3616,
      3614, 3612, 3610, 3608, 3606, 3604, 3602, 3600, 3598, 3596, 3594, 3592, 3590, 3587, 3585, 3583, 3581, 3579, 3577, 3575,
      3573, 3571, 3569, 3567, 3564, 3562, 3560, 3558, 3556, 3554, 3552, 3550, 3547, 3545, 3543, 3541, 3539, 3537, 3535, 3532,
      3530, 3528, 3526, 3524, 3522, 3519, 3517, 3515, 3513, 3511, 3508, 3506, 3504, 3502, 3500, 3497, 3495, 3493, 3491, 3488,
      3486, 3484, 3482, 3480, 3477, 3475, 3473, 3471, 3468, 3466, 3464, 3461, 3459, 3457, 3455, 3452, 3450, 3448, 3445, 3443,
      3441, 3439, 3436, 3434, 3432, 3429, 3427, 3425, 3422, 3420, 3418, 3415, 3413, 3411, 3408, 3406, 3404, 3401, 3399, 3397,
      3394, 3392, 3389, 3387, 3385, 3382, 3380, 3378, 3375, 3373, 3370, 3368, 3366, 3363, 3361, 3358, 3356, 3354, 3351, 3349,
      3346, 3344, 3341, 3339, 3336, 3334, 3332, 3329, 3327, 3324, 3322, 3319, 3317, 3314, 3312, 3309, 3307, 3305, 3302, 3300,
      3297, 3295, 3292, 3290, 3287, 3285, 3282, 3280, 3277, 3275, 3272, 3270, 3267, 3264, 3262, 3259, 3257, 3254, 3252, 3249,
      3247, 3244, 3242, 3239, 3237, 3234, 3231, 3229, 3226, 3224, 3221, 3219, 3216, 3213, 3211, 3208, 3206, 3203, 3200, 3198,
      3195, 3193, 3190, 3187, 3185, 3182, 3180, 3177, 3174, 3172, 3169, 3166, 3164, 3161, 3159, 3156, 3153, 3151, 3148, 3145,
      3143, 3140, 3137, 3135, 3132, 3129, 3127, 3124, 3121, 3119, 3116, 3113, 3111, 3108, 3105, 3103, 3100, 3097, 3094, 3092,
      3089, 3086, 3084, 3081, 3078, 3076, 3073, 3070, 3067, 3065, 3062, 3059, 3056, 3054, 3051, 3048, 3046, 3043, 3040, 3037,
      3035, 3032, 3029, 3026, 3023, 3021, 3018, 3015, 3012, 3010, 3007, 3004, 3001, 2999, 2996, 2993, 2990, 2987, 2985, 2982,
      2979, 2976, 2973, 2971, 2968, 2965, 2962, 2959, 2957, 2954, 2951, 2948, 2945, 2942, 2940, 2937, 2934, 2931, 2928, 2925,
      2923, 2920, 2917, 2914, 2911, 2908, 2906, 2903, 2900, 2897, 2894, 2891, 2888, 2886, 2883, 2880, 2877, 2874, 2871, 2868,
      2865, 2863, 2860, 2857, 2854, 2851, 2848, 2845, 2842, 2839, 2837, 2834, 2831, 2828, 2825, 2822, 2819, 2816, 2813, 2810,
      2807, 2805, 2802, 2799, 2796, 2793, 2790, 2787, 2784, 2781, 2778, 2775, 2772, 2769, 2766, 2764, 2761, 2758, 2755, 2752,
      2749, 2746, 2743, 2740, 2737, 2734, 2731, 2728, 2725, 2722, 2719, 2716, 2713, 2710, 2707, 2704, 2701, 2698, 2695, 2692,
      2689, 2686, 2683, 2680, 2677, 2674, 2671, 2669, 2666, 2663, 2660, 2657, 2654, 2651, 2648, 2645, 2642, 2638, 2635, 2632,
      2629, 2626, 2623, 2620, 2617, 2614, 2611, 2608, 2605, 2602, 2599, 2596, 2593, 2590, 2587, 2584, 2581, 2578, 2575, 2572,
      2569, 2566, 2563, 2560, 2557, 2554, 2551, 2548, 2545, 2542, 2539, 2535, 2532, 2529, 2526, 2523, 2520, 2517, 2514, 2511,
      2508, 2505, 2502, 2499, 2496, 2493, 2490, 2487, 2483, 2480, 2477, 2474, 2471, 2468, 2465, 2462, 2459, 2456, 2453, 2450,
      2447, 2443, 2440, 2437, 2434, 2431, 2428, 2425, 2422, 2419, 2416, 2413, 2410, 2406, 2403, 2400, 2397, 2394, 2391, 2388,
      2385, 2382, 2379, 2375, 2372, 2369, 2366, 2363, 2360, 2357, 2354, 2351, 2348, 2344, 2341, 2338, 2335, 2332, 2329, 2326,
      2323, 2320, 2316, 2313, 2310, 2307, 2304, 2301, 2298, 2295, 2291, 2288, 2285, 2282, 2279, 2276, 2273, 2270, 2266, 2263,
      2260, 2257, 2254, 2251, 2248, 2245, 2241, 2238, 2235, 2232, 2229, 2226, 2223, 2220, 2216, 2213, 2210, 2207, 2204, 2201,
      2198, 2195, 2191, 2188, 2185, 2182, 2179, 2176, 2173, 2169, 2166, 2163, 2160, 2157, 2154, 2151, 2147, 2144, 2141, 2138,
      2135, 2132, 2129, 2126, 2122, 2119, 2116, 2113, 2110, 2107, 2104, 2100, 2097, 2094, 2091, 2088, 2085, 2082, 2078, 2075,
      2072, 2069, 2066, 2063, 2060, 2056, 2053, 2050, 2047, 2044, 2041, 2038, 2034, 2031, 2028, 2025, 2022, 2019, 2016, 2012,
      2009, 2006, 2003, 2000, 1997, 1994, 1990, 1987, 1984, 1981, 1978, 1975, 1972, 1968, 1965, 1962, 1959, 1956, 1953, 1950,
      1947, 1943, 1940, 1937, 1934, 1931, 1928, 1925, 1921, 1918, 1915, 1912, 1909, 1906, 1903, 1899, 1896, 1893, 1890, 1887,
      1884, 1881, 1878, 1874, 1871, 1868, 1865, 1862, 1859, 1856, 1853, 1849, 1846, 1843, 1840, 1837, 1834, 1831, 1828, 1824,
      1821, 1818, 1815, 1812, 1809, 1806, 1803, 1799, 1796, 1793, 1790, 1787, 1784, 1781, 1778, 1774, 1771, 1768, 1765, 1762,
      1759, 1756, 1753, 1750, 1746, 1743, 1740, 1737, 1734, 1731, 1728, 1725, 1722, 1719, 1715, 1712, 1709, 1706, 1703, 1700,
      1697, 1694, 1691, 1688, 1684, 1681, 1678, 1675, 1672, 1669, 1666, 1663, 1660, 1657, 1654, 1651, 1647, 1644, 1641, 1638,
      1635, 1632, 1629, 1626, 1623, 1620, 1617, 1614, 1611, 1607, 1604, 1601, 1598, 1595, 1592, 1589, 1586, 1583, 1580, 1577,
      1574, 1571, 1568, 1565, 1562, 1559, 1555, 1552, 1549, 1546, 1543, 1540, 1537, 1534, 1531, 1528, 1525, 1522, 1519, 1516,
      1513, 1510, 1507, 1504, 1501, 1498, 1495, 1492, 1489, 1486, 1483, 1480, 1477, 1474, 1471, 1468, 1465, 1462, 1459, 1456,
      1452, 1449, 1446, 1443, 1440, 1437, 1434, 1431, 1428, 1425, 1423, 1420, 1417, 1414, 1411, 1408, 1405, 1402, 1399, 1396,
      1393, 1390, 1387, 1384, 1381, 1378, 1375, 1372, 1369, 1366, 1363, 1360, 1357, 1354, 1351, 1348, 1345, 1342, 1339, 1336,
      1333, 1330, 1328, 1325, 1322, 1319, 1316, 1313, 1310, 1307, 1304, 1301, 1298, 1295, 1292, 1289, 1287, 1284, 1281, 1278,
      1275, 1272, 1269, 1266, 1263, 1260, 1257, 1255, 1252, 1249, 1246, 1243, 1240, 1237, 1234, 1231, 1229, 1226, 1223, 1220,
      1217, 1214, 1211, 1208, 1206, 1203, 1200, 1197, 1194, 1191, 1188, 1186, 1183, 1180, 1177, 1174, 1171, 1169, 1166, 1163,
      1160, 1157, 1154, 1152, 1149, 1146, 1143, 1140, 1137, 1135, 1132, 1129, 1126, 1123, 1121, 1118, 1115, 1112, 1109, 1107,
      1104, 1101, 1098, 1095, 1093, 1090, 1087, 1084, 1082, 1079, 1076, 1073, 1071, 1068, 1065, 1062, 1059, 1057, 1054, 1051,
      1048, 1046, 1043, 1040, 1038, 1035, 1032, 1029, 1027, 1024, 1021, 1018, 1016, 1013, 1010, 1008, 1005, 1002, 1000,  997,
      994,  991,  989,  986,  983,  981,  978,  975,  973,  970,  967,  965,  962,  959,  957,  954,  951,  949,  946,  943,
      941,  938,  935,  933,  930,  928,  925,  922,  920,  917,  914,  912,  909,  907,  904,  901,  899,  896,  894,  891,
      888,  886,  883,  881,  878,  875,  873,  870,  868,  865,  863,  860,  857,  855,  852,  850,  847,  845,  842,  840,
      837,  835,  832,  830,  827,  824,  822,  819,  817,  814,  812,  809,  807,  804,  802,  799,  797,  794,  792,  789,
      787,  785,  782,  780,  777,  775,  772,  770,  767,  765,  762,  760,  758,  755,  753,  750,  748,  745,  743,  740,
      738,  736,  733,  731,  728,  726,  724,  721,  719,  716,  714,  712,  709,  707,  705,  702,  700,  697,  695,  693,
      690,  688,  686,  683,  681,  679,  676,  674,  672,  669,  667,  665,  662,  660,  658,  655,  653,  651,  649,  646,
      644,  642,  639,  637,  635,  633,  630,  628,  626,  623,  621,  619,  617,  614,  612,  610,  608,  606,  603,  601,
      599,  597,  594,  592,  590,  588,  586,  583,  581,  579,  577,  575,  572,  570,  568,  566,  564,  562,  559,  557,
      555,  553,  551,  549,  547,  544,  542,  540,  538,  536,  534,  532,  530,  527,  525,  523,  521,  519,  517,  515,
      513,  511,  509,  507,  504,  502,  500,  498,  496,  494,  492,  490,  488,  486,  484,  482,  480,  478,  476,  474,
      472,  470,  468,  466,  464,  462,  460,  458,  456,  454,  452,  450,  448,  446,  444,  442,  440,  438,  436,  434,
      432,  431,  429,  427,  425,  423,  421,  419,  417,  415,  413,  411,  410,  408,  406,  404,  402,  400,  398,  396,
      395,  393,  391,  389,  387,  385,  384,  382,  380,  378,  376,  374,  373,  371,  369,  367,  365,  364,  362,  360,
      358,  356,  355,  353,  351,  349,  348,  346,  344,  342,  341,  339,  337,  335,  334,  332,  330,  329,  327,  325,
      323,  322,  320,  318,  317,  315,  313,  312,  310,  308,  307,  305,  303,  302,  300,  298,  297,  295,  294,  292,
      290,  289,  287,  286,  284,  282,  281,  279,  278,  276,  274,  273,  271,  270,  268,  267,  265,  264,  262,  260,
      259,  257,  256,  254,  253,  251,  250,  248,  247,  245,  244,  242,  241,  239,  238,  236,  235,  233,  232,  231,
      229,  228,  226,  225,  223,  222,  221,  219,  218,  216,  215,  213,  212,  211,  209,  208,  207,  205,  204,  202,
      201,  200,  198,  197,  196,  194,  193,  192,  190,  189,  188,  186,  185,  184,  182,  181,  180,  179,  177,  176,
      175,  173,  172,  171,  170,  168,  167,  166,  165,  163,  162,  161,  160,  159,  157,  156,  155,  154,  152,  151,
      150,  149,  148,  147,  145,  144,  143,  142,  141,  140,  139,  137,  136,  135,  134,  133,  132,  131,  130,  128,
      127,  126,  125,  124,  123,  122,  121,  120,  119,  118,  117,  116,  115,  113,  112,  111,  110,  109,  108,  107,
      106,  105,  104,  103,  102,  101,  100,   99,   98,   97,   97,   96,   95,   94,   93,   92,   91,   90,   89,   88,
      87,   86,   85,   84,   84,   83,   82,   81,   80,   79,   78,   77,   77,   76,   75,   74,   73,   72,   72,   71,
      70,   69,   68,   67,   67,   66,   65,   64,   63,   63,   62,   61,   60,   60,   59,   58,   57,   57,   56,   55,
      54,   54,   53,   52,   52,   51,   50,   49,   49,   48,   47,   47,   46,   45,   45,   44,   43,   43,   42,   41,
      41,   40,   40,   39,   38,   38,   37,   37,   36,   35,   35,   34,   34,   33,   32,   32,   31,   31,   30,   30,
      29,   29,   28,   28,   27,   27,   26,   26,   25,   25,   24,   24,   23,   23,   22,   22,   21,   21,   20,   20,
      19,   19,   18,   18,   18,   17,   17,   16,   16,   16,   15,   15,   14,   14,   14,   13,   13,   13,   12,   12,
      11,   11,   11,   10,   10,   10,    9,    9,    9,    9,    8,    8,    8,    7,    7,    7,    7,    6,    6,    6,
      6,    5,    5,    5,    5,    4,    4,    4,    4,    3,    3,    3,    3,    3,    2,    2,    2,    2,    2,    2,
      1,    1,    1,    1,    1,    1,    1,    1,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    1,    1,    1,
      1,    1,    1,    1,    1,    2,    2,    2,    2,    2,    2,    3,    3,    3,    3,    3,    4,    4,    4,    4,
      5,    5,    5,    5,    6,    6,    6,    6,    7,    7,    7,    7,    8,    8,    8,    9,    9,    9,    9,   10,
      10,   10,   11,   11,   11,   12,   12,   13,   13,   13,   14,   14,   14,   15,   15,   16,   16,   16,   17,   17,
      18,   18,   18,   19,   19,   20,   20,   21,   21,   22,   22,   23,   23,   24,   24,   25,   25,   26,   26,   27,
      27,   28,   28,   29,   29,   30,   30,   31,   31,   32,   32,   33,   34,   34,   35,   35,   36,   37,   37,   38,
      38,   39,   40,   40,   41,   41,   42,   43,   43,   44,   45,   45,   46,   47,   47,   48,   49,   49,   50,   51,
      52,   52,   53,   54,   54,   55,   56,   57,   57,   58,   59,   60,   60,   61,   62,   63,   63,   64,   65,   66,
      67,   67,   68,   69,   70,   71,   72,   72,   73,   74,   75,   76,   77,   77,   78,   79,   80,   81,   82,   83,
      84,   84,   85,   86,   87,   88,   89,   90,   91,   92,   93,   94,   95,   96,   97,   97,   98,   99,  100,  101,
      102,  103,  104,  105,  106,  107,  108,  109,  110,  111,  112,  113,  115,  116,  117,  118,  119,  120,  121,  122,
      123,  124,  125,  126,  127,  128,  130,  131,  132,  133,  134,  135,  136,  137,  139,  140,  141,  142,  143,  144,
      145,  147,  148,  149,  150,  151,  152,  154,  155,  156,  157,  159,  160,  161,  162,  163,  165,  166,  167,  168,
      170,  171,  172,  173,  175,  176,  177,  179,  180,  181,  182,  184,  185,  186,  188,  189,  190,  192,  193,  194,
      196,  197,  198,  200,  201,  202,  204,  205,  207,  208,  209,  211,  212,  213,  215,  216,  218,  219,  221,  222,
      223,  225,  226,  228,  229,  231,  232,  233,  235,  236,  238,  239,  241,  242,  244,  245,  247,  248,  250,  251,
      253,  254,  256,  257,  259,  260,  262,  264,  265,  267,  268,  270,  271,  273,  274,  276,  278,  279,  281,  282,
      284,  286,  287,  289,  290,  292,  294,  295,  297,  298,  300,  302,  303,  305,  307,  308,  310,  312,  313,  315,
      317,  318,  320,  322,  323,  325,  327,  329,  330,  332,  334,  335,  337,  339,  341,  342,  344,  346,  348,  349,
      351,  353,  355,  356,  358,  360,  362,  364,  365,  367,  369,  371,  373,  374,  376,  378,  380,  382,  384,  385,
      387,  389,  391,  393,  395,  396,  398,  400,  402,  404,  406,  408,  410,  411,  413,  415,  417,  419,  421,  423,
      425,  427,  429,  431,  432,  434,  436,  438,  440,  442,  444,  446,  448,  450,  452,  454,  456,  458,  460,  462,
      464,  466,  468,  470,  472,  474,  476,  478,  480,  482,  484,  486,  488,  490,  492,  494,  496,  498,  500,  502,
      504,  507,  509,  511,  513,  515,  517,  519,  521,  523,  525,  527,  530,  532,  534,  536,  538,  540,  542,  544,
      547,  549,  551,  553,  555,  557,  559,  562,  564,  566,  568,  570,  572,  575,  577,  579,  581,  583,  586,  588,
      590,  592,  594,  597,  599,  601,  603,  606,  608,  610,  612,  614,  617,  619,  621,  623,  626,  628,  630,  633,
      635,  637,  639,  642,  644,  646,  649,  651,  653,  655,  658,  660,  662,  665,  667,  669,  672,  674,  676,  679,
      681,  683,  686,  688,  690,  693,  695,  697,  700,  702,  705,  707,  709,  712,  714,  716,  719,  721,  724,  726,
      728,  731,  733,  736,  738,  740,  743,  745,  748,  750,  753,  755,  758,  760,  762,  765,  767,  770,  772,  775,
      777,  780,  782,  785,  787,  789,  792,  794,  797,  799,  802,  804,  807,  809,  812,  814,  817,  819,  822,  824,
      827,  830,  832,  835,  837,  840,  842,  845,  847,  850,  852,  855,  857,  860,  863,  865,  868,  870,  873,  875,
      878,  881,  883,  886,  888,  891,  894,  896,  899,  901,  904,  907,  909,  912,  914,  917,  920,  922,  925,  928,
      930,  933,  935,  938,  941,  943,  946,  949,  951,  954,  957,  959,  962,  965,  967,  970,  973,  975,  978,  981,
      983,  986,  989,  991,  994,  997, 1000, 1002, 1005, 1008, 1010, 1013, 1016, 1018, 1021, 1024, 1027, 1029, 1032, 1035,
      1038, 1040, 1043, 1046, 1048, 1051, 1054, 1057, 1059, 1062, 1065, 1068, 1071, 1073, 1076, 1079, 1082, 1084, 1087, 1090,
      1093, 1095, 1098, 1101, 1104, 1107, 1109, 1112, 1115, 1118, 1121, 1123, 1126, 1129, 1132, 1135, 1137, 1140, 1143, 1146,
      1149, 1152, 1154, 1157, 1160, 1163, 1166, 1169, 1171, 1174, 1177, 1180, 1183, 1186, 1188, 1191, 1194, 1197, 1200, 1203,
      1206, 1208, 1211, 1214, 1217, 1220, 1223, 1226, 1229, 1231, 1234, 1237, 1240, 1243, 1246, 1249, 1252, 1255, 1257, 1260,
      1263, 1266, 1269, 1272, 1275, 1278, 1281, 1284, 1287, 1289, 1292, 1295, 1298, 1301, 1304, 1307, 1310, 1313, 1316, 1319,
      1322, 1325, 1328, 1330, 1333, 1336, 1339, 1342, 1345, 1348, 1351, 1354, 1357, 1360, 1363, 1366, 1369, 1372, 1375, 1378,
      1381, 1384, 1387, 1390, 1393, 1396, 1399, 1402, 1405, 1408, 1411, 1414, 1417, 1420, 1423, 1425, 1428, 1431, 1434, 1437,
      1440, 1443, 1446, 1449, 1452, 1456, 1459, 1462, 1465, 1468, 1471, 1474, 1477, 1480, 1483, 1486, 1489, 1492, 1495, 1498,
      1501, 1504, 1507, 1510, 1513, 1516, 1519, 1522, 1525, 1528, 1531, 1534, 1537, 1540, 1543, 1546, 1549, 1552, 1555, 1559,
      1562, 1565, 1568, 1571, 1574, 1577, 1580, 1583, 1586, 1589, 1592, 1595, 1598, 1601, 1604, 1607, 1611, 1614, 1617, 1620,
      1623, 1626, 1629, 1632, 1635, 1638, 1641, 1644, 1647, 1651, 1654, 1657, 1660, 1663, 1666, 1669, 1672, 1675, 1678, 1681,
      1684, 1688, 1691, 1694, 1697, 1700, 1703, 1706, 1709, 1712, 1715, 1719, 1722, 1725, 1728, 1731, 1734, 1737, 1740, 1743,
      1746, 1750, 1753, 1756, 1759, 1762, 1765, 1768, 1771, 1774, 1778, 1781, 1784, 1787, 1790, 1793, 1796, 1799, 1803, 1806,
      1809, 1812, 1815, 1818, 1821, 1824, 1828, 1831, 1834, 1837, 1840, 1843, 1846, 1849, 1853, 1856, 1859, 1862, 1865, 1868,
      1871, 1874, 1878, 1881, 1884, 1887, 1890, 1893, 1896, 1899, 1903, 1906, 1909, 1912, 1915, 1918, 1921, 1925, 1928, 1931,
      1934, 1937, 1940, 1943, 1947, 1950, 1953, 1956, 1959, 1962, 1965, 1968, 1972, 1975, 1978, 1981, 1984, 1987, 1990, 1994,
      1997, 2000, 2003, 2006, 2009, 2012, 2016, 2019, 2022, 2025, 2028, 2031, 2034, 2038, 2041, 2044
};


// Frequenztabelle fr Midi Noten-Nr.
const uint16_t midi_ton[] PROGMEM = {
	8,8,9,9,10,10,11,12,12,13,14,15,								// Key-Nr.:   0 - 11
	16,17,18,19,20,21,23,24,25,27,29,30,							// Key-Nr.:  12 - 23
	32,34,36,38,41,43,46,48,51,55,58,61,							// Key-Nr.:  24 - 35
	65,69,73,77,82,87,92,97,103,110,116,123,						// Key-Nr.:  36 - 47
	130,138,146,155,164,174,184,195,207,220,233,246,				// Key-Nr.:  48 - 59
	261,277,293,311,329,349,369,391,415,440,466,493,				// Key-Nr.:  60 - 71
	523,554,587,622,659,698,739,783,830,880,932,987,				// Key-Nr.:  72 - 83
	1046,1108,1174,1244,1318,1396,1479,1567,1661,1760,1864,1975,	// Key-Nr.:  84 - 95
	2093,2217,2349,2489,2637,2793,2959,3135,3322,3520,3729,3951,	// Key-Nr.:  96 - 107
	4186,4434,4698,4978,5274,5587,5919,6271,6644,7040,7458,7902,	// Key-Nr.: 108 - 119
	8372,8869,9397,9956,10548,11175,11839,12543						// Key-Nr.: 120 - 127
};

// Korrekturwerte fr genaue Frequenz der Midi Noten
const uint16_t midi_korr[] PROGMEM = {
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:   0 - 11
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  12 - 23
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  24 - 35
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  36 - 47
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  48 - 59
	0,0,0,0,0,0,0,0,0,200,0,0,								// Key-Nr.:  60 - 71
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  72 - 83
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  84 - 95
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.:  96 - 107
	0,0,0,0,0,0,0,0,0,0,0,0,								// Key-Nr.: 108 - 119
	0,0,0,0,0,0,0,0,0,0,0,0									// Key-Nr.: 120 - 127
};

//***********************************
// Set CPU-Clock 32MHz
//***********************************
void init_CLK(){
	OSC.CTRL = OSC_RC32MEN_bm;
	// Wait Oszillator is stable
	while ((OSC.STATUS & OSC_RC32MRDY_bm) == 0);
	// System Clock selection
	CCP = CCP_IOREG_gc;
	CLK.CTRL = CLK_SCLKSEL_RC32M_gc;
	// Auto Calibration
	DFLLRC32M.CTRL = DFLL_ENABLE_bm;
}

//****************************************
// init Timer_0
// ADSR-Hllkurve und LFO's
//****************************************
void init_TIMER0(){
	//TCC1.CTRLE = (1 << TC0_BYTEM_bp);       // Byte mode: 1 run as 8-bit counter
	// fclk/1
	TCC0.CTRLA |= TC_CLKSEL_DIV64_gc;
	// Timertopvalue
	TCC0.PER = 500;
	// High Level Interuppt
	TCC0.INTCTRLA = 0b00000001; // Interrupt Priotit?t Low
}

//***********************************
// init Timer_1 
// fr die Soundausgabe
//***********************************
void init_TIMER1(){
	//TCC1.CTRLE = (1 << TC0_BYTEM_bp);       // Byte mode: 1 run as 8-bit counter
	// fclk/1
	TCC1.CTRLA |= TC_CLKSEL_DIV1_gc;
	// Timertopvalue
	//TCC1.PER =  323; //  1us=31  5us=159  7us=225  10us=319 60us=1860 21usec=667
	TCC1.PER = 800; // 40KHz = ca. 25usec
	// High Level Interuppt
	TCC1.INTCTRLA = 0b00000011; // Interrupt Priorit?t High
}

//**************************************************
// init. PWM
// an PORT_D Auflsung 8Bit / 32kHz 
// Pin D0 = Filter-Frequenz
// Pin D1 = Filter-Resonanz
//**************************************************
void init_PWM()
{
	PORTD.DIR = 0xFF;
	PORTD.OUT = 0x00;
	TCD0.CTRLA = 0x03;			// divide clock by 4
	TCD0.PER = 0x00FF;			// 0xFFFF = 16Bit Resolution / 0x00FF = 8Bit Resolution
	//TCD0.CCA = 0x0000;		// A
	//TCD0.CCB = 0x0001;		// B
	//TCD0.CCC = 0x000F;		// C
	//TCD0.CCD = 0x000F;		// D
	TCD0.CTRLB = 0xF3;			// enable frequency generation for all 4 Channels
}

//***********************************************
// init DACA CHanal_0 for Sound
//***********************************************
void init_DACA(){
	// DAC Single Canal,Conversion Interval 1usec,Timing Control 16CLK, Ref.Volume 1.0V
	DACA.CTRLB = ~ (DAC_CHSEL_gm | DAC_CH0TRIG_bm) | DAC_CHSEL_SINGLE_gc;
	// Interne 1V Reference, right adjust (12 Bit);
	DACA.CTRLC = ~(DAC_REFSEL_gm | DAC_LEFTADJ_bm);
	// Adjust DAC Callbration Error
	DACA.GAINCAL = 127;
	// enabled DAC CH0
	DACA.CTRLA = (DAC_CH0EN_bm | DAC_ENABLE_bm);
}

//*******************************
// init Usart-Interface for Midi
//*******************************
void init_USART(){
	USARTF0.BAUDCTRLA = 0xC0;						// BSCALE = -6, BSEL = 4032
	USARTF0.BAUDCTRLB = 0xAF;						// ==> 31.250 Midi-Baudrate
	USARTF0.CTRLC = 0x03;							// Async, No Parity, 1 stop bit, 8 data bits
	USARTF0.CTRLB = 0x10;							// Enable RX
	USARTF0.CTRLA = 0x20;							// Enable RX interrupt
	USARTF0.CTRLA = USART_RXCINTLVL_MED_gc;			// RX interrupt middle level
}

//******************************
// init I/O Ports
//******************************
void init_PORT(){
	LED_DDR = 0xFF;									// Data direction for LED-Port
	PORTE.OUTSET= 0xFF;								// LED off
	PORTF.PIN2CTRL = 0x18;							// enabled Pullup on Midi-IN (RXD-Pin PortF2)
	PORTA.DIR |= (1<<PIN0);							// Testpin PA0 Datenrichtung auf Ausgang
}

//******************************
// Midi-Receive Interuppt
//******************************
ISR(USARTF0_RXC_vect){
	 midibuf[midibuf_wr_pointer] = USARTF0.DATA;	// Save Midi-Data to Buffer (Buffersize: 256 Byte)
	 midibuf_wr_pointer++;							// inc Buffer-Pointer
	 led_timer = 2;									// Midi-Led ON-Time
	 LED_OUT &= ~(1<<LED_0);						// Midi-Led ein
}

//*******************************
// Teste ob Midi-Daten im Buffer
//*******************************
void mididata_avail(){
	if(midibuf_wr_pointer != midibuf_rd_pointer)	// Wenn Schreib- und Lesezeiger des Buffers ungleich
	{midi_avail = 1;}								// dann sind Midi-Daten im Buffer (midi_avail=1).
	else{midi_avail = 0;}							// Wenn beide Zeiger gleich keine Midi-Daten vorhanden (midi_avail=0).
}

//*******************************
// read midi-data from buffer
// (Buffer-Size 256 Byte)
//*******************************
void read_mididata(){
	while(midibuf_wr_pointer == midibuf_rd_pointer);
	midi_data =  midibuf[midibuf_rd_pointer];
	midibuf_rd_pointer++;
}

//*******************************
// Play Midi-Note Chanal 1
//*******************************
void play_note1(){
	read_mididata();											// Midi Notenwert aus Buffer laden
	noten_nr = midi_data;										// Notenwert speichern
	frequenz = pgm_read_word (&(midi_ton[noten_nr]));			// Frequenzwert fr Midi-Note aus Tabelle laden
	korrekturwert = pgm_read_word (&(midi_korr[noten_nr]));		// Korrekturwert fr genau Frequenz der Midi-Note
	schrittweite1 = (__uint24)frequenz * 412;
	schrittweite2 = (__uint24)(frequenz+4) * 412;
	
	read_mididata();											// Midi Velocity-Wert holen
	tasten_flag1 = 1;
	adsr_phase1 = 1;
	adsr_phase1_f = 1;
	lfo1_phase = 1;
	lfo1_out = 0;
	lfo1_phaccu = 0;
	lfo1_sample_old = 0;
	if (lfo1_level>0)
	{
		lfo1_flag = 1;
	}
	
	lfo2_phase = 1;
	lfo2_out = 0;
	lfo2_phaccu = 0;
	lfo2_out_old = 0;
	if (lfo2_level>0)
	{
		lfo2_flag = 1;
	}
}

//*****************************
// Stop Midi-Note Cha1
//*****************************
void stop_note1(){
	read_mididata();		// read Midi Notenwert
	read_mididata();		// Velocity ingnorieren
	tasten_flag1 = 2;		// Tasten-Gate off
}

//--------------------------------------------------
// Change Sound-Prg.
//--------------------------------------------------
// Waveform-Selection
// ------------------
//   0 = Sinus/Sinus
//   1 = Sinus/Saw
//   2 = Saw/Sinus
//   3 = Saw/Saw
// 128 = FM
//--------------------------------------------------
void change_sound(){
	if (prgchange_nr == 0)
	{
		DcoWaveForm = 0;
	}
	if (prgchange_nr == 1)
	{
		DcoWaveForm = 1;
	}
	if (prgchange_nr == 2)
	{
		DcoWaveForm = 2;
	}
	if (prgchange_nr == 3)
	{
		DcoWaveForm = 3;
	}
	if (prgchange_nr == 4)
	{
		DcoWaveForm = 128;
	}
	if (prgchange_nr == 5)
	{
		DcoWaveForm = 64;
	}
}

//********************************
// Midi-Progamm-Change
//********************************
void midi_prgnr(){
	read_mididata();
	prgchange_nr = midi_data;
	change_sound();
}

//*****************************************************************
//          Midi-Controller-Daten auswerten
//-----------------------------------------------------------------
// Regler-Nr. 1-8 am Midi-Keyboard
// Regler-Nr. 1 und Modulationsrad haben gleiche Controller-Werte
// 
//*****************************************************************
void midi_ctrl(){
	
	read_mididata();
	switch (midi_data)
	{	
	//------------------------------
	// Regler-Nr.: 1
	// Modulation        (0xB0,0x01)
	//------------------------------
		case 0x01:
		read_mididata();
		
		break;
	//-------------------------------
	// Regler-Nr.: 2
	// Attack-Time DCA (0xB0,0x47)
	//-------------------------------
		case 0x47:
		read_mididata();
		attack_time = midi_data*(midi_data*2)+1;
		break;
	//------------------------------
	// Regler-Nr.: 3
	// Attack-Time VCF (0xB0,0x51)
	//------------------------------
		case 0x51:
		read_mididata();
		attack_time_f = midi_data*(midi_data*2)+1;
		break;
	//-------------------------------
	// Regler-Nr.: 4
	// Release-Time DCA
	// Release-Time VCF (0xB0,0x5B)
	//-------------------------------
		case 0x5B:
		read_mididata();
		release_time = midi_data*(midi_data*2)+2;
		release_time_f = midi_data*(midi_data*2)+2;
		break;
	//-------------------------------
	// Regler-Nr.: 5
	// LFO-Time DCO (0xB0,0x10)
	//-------------------------------
		case 0x10:
		read_mididata();
		lfo1_time = midi_data;
		lfo1_time = lfo1_time * lfo1_time; //((lfo1_time/20)+1);
		break;
	//-------------------------------
	// Regler-Nr.: 6
	// LFO-Level DCO (0xB0,0x50)
	//-------------------------------
		case 0x50:
		read_mididata();
		lfo1_level = midi_data;
		if (lfo1_level == 0)
		{
			lfo1_flag = 0;
		}
		else lfo1_flag = 1;
		break;
	//-------------------------------
	// Regler-Nr.: 7
	// LFO-Time VCF (0xB0,0x13)
	//-------------------------------
		case 0x13:
		read_mididata();
		lfo2_time = midi_data*5;
		// lfo3_time = midi_data*5;
		break;
	//-------------------------------
	// Regler-Nr.: 8
	// LFO-Level VCF (0xB0,0x02)
	//-------------------------------
		case 0x02:
		read_mididata();
	
		lfo2_level = midi_data;
		if (lfo2_level == 0)
		{
			lfo2_flag = 0;
		}
		else lfo2_flag = 1;
		/*
		 lfo3_level = midi_data;
		 if (lfo3_level == 0)
		 {
			 lfo3_flag = 0;
		 }
		 else lfo3_flag = 1;
		 */
		break;
	//---------------------------
	// Regler-Nr.: Fader
	// Filter-Resonanz (0xB0,0x07)
	//---------------------------
		case 0x07:
		read_mididata();
		resonace_f = midi_data*2;
		TCD0.CCB = 255 - resonace_f;
		break;
	}	
}

//********************************************
// init LFO's
//********************************************
init_LFO()
{
	//----------- LFO_1 DCO's ----------------
	lfo1_time  = 0;			// max 65535
	lfo1_level = 0;			// max 255
	lfo1_phase = 0;			//  
	lfo1_flag  = 0;			// Flag=1 LFO on
	
	//----------- LFO_2 VCF ------------------
	lfo2_time  = 0;			// max 65535
	lfo2_level = 0;			// max 255 
	lfo2_flag  = 0;			// Flag=1 LFO on
	
	//----------- LFO_3 DCA ------------------
	lfo3_time  = 0;			// max 65535 ms
	lfo3_level = 0;			// max 127
	lfo3_flag  = 0;			// Flag=1 LFO on
}

//********************************************
// init ADSR-Generator
//********************************************
void init_ADSR()
{
	//---------- ADSR DAC --------------------
	attack_level  = 65000;		// max. 65535
	attack_time   = 50;			// max. 65535
	sustain_level = 65000;		// max. 65535
	decay_time    = 0;		// max. 65535
	release_time  = 5000;		// max. 65535
	
	//---------- ADSR VCF --------------------
	attack_level_f  = 65000;	// max. 65535
	attack_time_f   = 125;		// max. 65535
	sustain_level_f = 65000;	// max. 65535
	decay_time_f    = 0;		// max. 65535
	release_time_f  = 5000;		// max. 65535
	resonace_f      = 0;		// max. 255
	TCD0.CCB = 255 - resonace_f;
}


//************************************************************************
//             Timer_0 Interruptroutine (alle 1msec)
//             -------------------------------------
//				    (ADSR-Generator und LFO's) 
//
// -----------------------------------------------------------------------
// tasten_flag  = 0  wenn kein Midi Notenwert empfangen wurde
// tasten_flag1 = 1  wenn "Midi-Note On"	 empfange wurde
// tasten_flag1 = 2  wenn "Midi-Note Off" empfangen wurde
// adsr_phase1  = 1-3 ADSR-Phasen
//************************************************************************
ISR(TCC0_OVF_vect ){
	
// ---------------- Tastendruck auswerten ------------------------------

if (tasten_flag1 == 1)				// Taste gedrckt ?
	{
	if (adsr_phase1 == 0)
		{
			adsr_phase1 = 1;		// Attack-Phase aktivieren
			adsr_phase1_f = 1;
			tasten_flag1 = 0;		// und Tasten-Flag lschen
		}
	}

else if (tasten_flag1 == 2)			// Taste losgelassen ?
	{
		adsr_phase1 = 3;			// Release-Phase aktivieren
		adsr_phase1_f = 3;
		tasten_flag1 = 0;			// und Tasten-Flag lschen
	}
	
// --------------- DCA Attack Phasen_1  -----------------------
	
if (adsr_phase1 == 1)
	{
	  if (adsr_out < attack_level-attack_time)
	    {
		  adsr_out = adsr_out + ((attack_level - adsr_out ) / attack_time);
		  if (adsr_out >= (attack_level - (attack_level/5)))
		  {
			  adsr_phase1 = 2;
		  }
		}
	}
		
// ---------------- DCA Decay Phase_2 -------------------------			

if (adsr_phase1 == 2)
	{
		if (adsr_out > sustain_level+decay_time)
			{
				adsr_out = adsr_out -((adsr_out -sustain_level) / decay_time);
			}
	}
 
 // --------------- DCA Release Phase_3 -----------------------

if (adsr_phase1 == 3)
	{
		if (adsr_out > 0)
		 {
			 adsr_out = adsr_out - ((adsr_out / release_time)+1);
		 }
		 else
		 {
			 adsr_phase1 = 0;
			 adsr_phase1_f = 0;
			 TCD0.CCA = 0;			// Filter off 
			 lfo1_flag = 0;
			 lfo2_flag = 0;
		 }
	 }	  

// --------------- VCF Attack Phasen_1 ------------------------

if (adsr_phase1_f == 1)
{
	if (adsr_out_f < attack_level_f-attack_time_f)
	{
		adsr_out_f = adsr_out_f + ((attack_level_f - adsr_out_f ) / attack_time_f);
		if (adsr_out_f >= (attack_level_f - (attack_level_f/5)))
		{
			adsr_phase1_f = 2;
		}
		//TCD0.CCA = adsr_out_f >> 8;			// PWM Port D0 Filterfrequenz 8Bit
	}
}

// ---------------- VCF Decay Phase_2 -------------------------

if (adsr_phase1_f == 2)
{
	if (adsr_out_f > sustain_level_f+decay_time_f)
	{
		adsr_out_f = adsr_out_f -((adsr_out_f -sustain_level_f) / decay_time_f);
		//TCD0.CCA = adsr_out_f >> 8;			// PWM Port D0 Filterfrequenz 8Bit
		//TCD0.CCA = adsr_out_f >>8;
	}
}

// ---------------- VCF Release Phase_3 -----------------------

if (adsr_phase1_f == 3)
{
	if (adsr_out_f > 0)
	{
		adsr_out_f = adsr_out_f - ((adsr_out_f/ release_time_f)+1);
		//TCD0.CCA = adsr_out_f >> 8;			// PWM Port D0 Filterfrequenz 8Bit
	}
	else
	{
		adsr_phase1_f = 0;
	}
}

 //------------------------------------------------------
 //                     LFO_1 - DCO
 //------------------------------------------------------
 if (lfo1_flag == 1)
 {
	 lfo1_phaccu = lfo1_phaccu + lfo1_time;
	 lfo1_sample = pgm_read_byte (&(triangle[lfo1_phaccu>>8]));
	 
	 //--------------- LFO - DCO Mix -----------------
	 
	 __uint24 lfo1_temp = 0;
	 lfo1_temp = lfo1_level*((lfo1_level/25)+1);
	 lfo1_out = (lfo1_sample * lfo1_temp);
	
	//---------------- Waveform pos/neg Phase --------
	 
	 switch (lfo1_phase){
		 case 1:
		 if (lfo1_sample < lfo1_sample_old)
		 {
			 lfo1_phase = 2;
		 }
		 else
		 {
			 schrittweite1 = (__uint24)frequenz * 412 + lfo1_out;
		 }
		 break;

		 case 2:
		 if (lfo1_sample > lfo1_sample_old)
		 {
			 lfo1_phase = 3;
		 }
		 else
		 {
			 schrittweite1 = (__uint24)frequenz * 412 +  lfo1_out;
		 }
		 break;
		 
		 case 3:
		 if (lfo1_sample < lfo1_sample_old)
		 {
			 lfo1_phase = 4;
		 }
		 else
		 {
			 schrittweite1 = (__uint24)frequenz * 412 - lfo1_out;
		 }
		 break;

		 case 4:
		 if (lfo1_sample > lfo1_sample_old)
		 {
			 lfo1_phase = 1;
		 }
		 else
		 {
			 schrittweite1 = (__uint24)frequenz * 412 - lfo1_out;
		 }
		 break;
	 }
	 
	 lfo1_sample_old = lfo1_sample;	// neuer LFO-Wert speichern
 }

//------------------------------------------------------
//                     LFO_2 - VCF
//------------------------------------------------------
//lfo2_flag = 0;
if (lfo2_flag == 1)
{
	lfo2_phaccu = lfo2_phaccu + lfo2_time;
	lfo2_out = pgm_read_byte (&(triangle[lfo2_phaccu>>8]));
	lfo2_out = 255-(lfo2_out*(lfo2_level<<1)/255);
}
else lfo2_out = 255;

//------------------------------------------------------
//                 ADSR + LFO_2 Mix to CV-VCF
//------------------------------------------------------

	TCD0.CCA = ((uint16_t)(adsr_out_f >> 8) * lfo2_out >> 8);


//------------------------------------------------------
//                     LFO_3 - DCA
//------------------------------------------------------
lfo3_flag = 0;
if (lfo3_flag == 1)
{
	lfo3_phaccu = lfo3_phaccu + lfo3_time;
	lfo3_out = pgm_read_byte (&(triangle[lfo3_phaccu>>8]));
	lfo3_out = 255-(lfo3_out*(lfo3_level<<1)/255);
	lfo3_out = lfo3_out << 4;
}
else lfo3_out = 4095;

//------------------------------------------------------
//                 ADSR + LFO_3 Mix to DCA
//------------------------------------------------------
 
   adsr_1 = ((uint32_t)(adsr_out >> 4) * lfo3_out >>12);
 
//------------------------------------------
// Status-Led fr Midi-Empfang ausschalten
//------------------------------------------
if (led_timer == 0)
	{
		LED_OUT |= (1<<LED_0);
	}
	else led_timer--;
}

 
  
//**************************************************************
//                       Main-Programm
//**************************************************************
int main(void)
{
	init_CLK();		// init CPU-Clock to 32MHz
	init_USART();	// init usart for midi
	init_PORT();	// init ports
	init_TIMER0();  // init ADSR-Timer
	init_TIMER1();	// init Timer1 fr Soundausgabe
	init_DACA();	// init DACA Chanal_0
	init_PWM();		// init HW-PWM fr Filter-Steuerung
	init_ADSR();	// ADSR-Parameter auf Default
	init_LFO();		// init LFO's
	 
	PMIC.CTRL |= PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;     //  Interruptlevel aktiviert
	sei();
	
	sample1 = pgm_read_word (&(sinus_12Bit[phaccu_x]));	// einen Dummy Tabellenzugriff wegen Optimierung des Compilers
	
	prgchange_nr = 0;	// Sound-Nr.3 = Sinus;
	change_sound();
	
	 
	//-------------------- Main-Loop -------------------------

	while(1)
	{		
		mididata_avail();
			if (midi_avail == 1)
			{
				read_mididata();
				
				switch (midi_data)
				{
					case 0x90: play_note1();
					break;
					case 0x80: stop_note1();
					break;
					case 0xB0: midi_ctrl();
					break;
					case 0xC0: midi_prgnr();
					break;
				}						
			}		
	}
} 






//****************************************************************************************************************
//
//                                   nicht verwendete Routinen
//
//****************************************************************************************************************
/*	
	vol = adsr_1;									// Lautstrke laden
	sample = (((int16_t)sample-128)*vol+ 128*255);	// Ausgabewert (16Bit) mit Spannungsoffset 1/2 URef berechnen


// PORTD (Tastenport) auf Eingang schalten
KEY_DDR &= ~ALL_KEYS;
// Pullup-Widerstnde an PORTD fr Tastenabfrage aktivieren
KEY_PORT.PIN1CTRL = 0x38; // PULLUP an SW0

// LED_OUT ^= (1<<LED_0);

if (PORTD.IN & 0x02)
{
	LED_OUT |= (1<<LED_1);	// Led_1 aus
	table_ch1 = 0;
}
else
{
	LED_OUT &= ~(1<<LED_1);	// Led_1 ein
	table_ch1 =1;
}

PORTA.OUT |= (1<<PIN0);   // Portbit High
PORTA.OUT &= ~ (1<<PIN0); // Portbit Low

//***************************************************
// Union-Zugriff auf low and high Word
// word = 32Bit Variable
// pha.phaccu_l = untere 16Bit Variable
// pha.phaccu_h =  obere 16Bit Variable
//***************************************************
typedef uint32_t word;
union _word {
	word _word;
	struct
	{
		uint16_t phaccu_l;
		uint16_t phaccu_h;
	};
} pha;

if (phaccu1 > 800000)
{
	LED_OUT &= ~(1<<LED_1);	// Led_1 ein
	LED_OUT |= (1<<LED_1);
}


/------------------------------------------------------------------------
// Soudausgabe
// Timer1 Interruptroutine: alle 25usec = 40.0 KHz Samplerate
//------------------------------------------------------------------------
ISR(TCC1_OVF_vect ){
		
	sound_out();

	// DCO_1 (Zeit 1.8sec)
	//phaccu1 = phaccu1 + schrittweite1;						// 24Bit Phasenaccu berechnen	
	//phaccu_x= phaccu1;
	//phaccu_x = phaccu_x >>2;								// Die oberen 12Bit von phaccu1 fr 

Tabellenzugriff
	//phaccu_x = phaccu_x >>8;
	//phaccu_x = phaccu_x >>2;
	sample1 = pgm_read_word (&(sinus_12Bit[phaccu_x]));		// Phasenwert aus 12Bit Sinus-Tabelle laden
	DACA.CH0DATA = sample1;
	
	PORTA.OUT |= (1<<PIN0);   // Portbit High

	// DCO_2 (Zeit 1.8sec)
	phaccu2 = phaccu2 + schrittweite2;
	phaccu_x = phaccu2;
	phaccu_x = phaccu_x >>2;
	phaccu_x = phaccu_x >>8;
	phaccu_x = phaccu_x >>2;
	sample2 = pgm_read_word (&(sinus_12Bit[phaccu_x]));
	
	// Aplitudenberechnung und DAC Ausgabe (Zeit 2.4sec)
	sample_x = sample1 + sample2;
	sample_x = sample_x * adsr_1;
	sample_x = sample_x >> 2;
	sample_x = sample_x >> 8;
	sample_x = sample_x >> 2;
	sample_x = sample_x >> 1;
	DACA.CH0DATA = sample_x;
	
	
	
	//PORTA.OUT &= ~ (1<<PIN0);   // Portbit low
	//PORTA.OUT |= (1<<PIN0);   // Portbit High
	
	
	
//init portc for spi
void spi_port_init(void){
	PORTC.DIRSET  = (1 << SPI_MOSI) | (1 << SPI_SCK) | (1<<SPI_SS) | (1<<SPI_LDAC);
	PORTC.OUTSET = (1 << SPI_SS);			// SPI_SS Pin auf high
	PORTC.OUTSET = (1 << SPI_LDAC);
}

// init spi-Register (spi-clock 2Mhz)
void spi_init(void){
	SPIC.CTRL =  SPI_MODE_0_gc | SPI_PRESCALER_DIV4_gc | (1<< SPI_ENABLE_bp) | (1<< SPI_MASTER_bp) | (1<< SPI_CLK2X_bp);
	SPIC.INTCTRL = SPI_INTLVL_OFF_gc; // spi-interuppt disabled
}

// init Timer3 (Interuppt for ADSR 1msec)
void timerE0_init(){
	//TCC1.CTRLE = (1 << TC0_BYTEM_bp);       // Byte mode: 1 run as 8-bit counter
	// fclk/1
	TCE0.CTRLA |= TC_CLKSEL_DIV64_gc;
	// Timertopvalue
	TCE0.PER = 500; //500 ;
	// High Level Interuppt
	TCE0.INTCTRLA = 0b00000001; // Interrupt Priotit?t Low
}

// --------------------------
	// Regler-Nr.: X
	// Frequenz-Tune (0xB0,0x01)
	//---------------------------
	
	if (midi_data == 0x01)
	{
		frequenz = pgm_read_word (&(midi_ton[noten_nr]));
		read_mididata();
		frq_tune = midi_data;
		if (frq_tune <=64)
		{
			frequenz = frequenz-(64-frq_tune);	
		}
		else
		{
			frequenz = frequenz+(frq_tune-64);
		}
		return;
	}
	

*/







